import React, { useEffect, useRef, useState } from 'react';
import { Box, Typography, Paper, Avatar, Collapse, IconButton } from '@mui/material';
import { Loader2, ChevronDown, ChevronUp } from 'lucide-react';
import ReactMarkdown from 'react-markdown';
import rehypeHighlight from 'rehype-highlight';
import 'highlight.js/styles/github.css';
import type { DocumentMetadata } from '../../../electron/types';

interface ChatMessage {
  id: string;
  text: string;
  isUser: boolean;
  timestamp: string;
  sources?: DocumentMetadata[];
  avatar?: string;
}

interface MessageListProps {
  messages: ChatMessage[];
}

interface ThinkingContentProps {
  content: string;
  isThinking: boolean;
}

interface MessageSegment {
  type: 'text' | 'thinking';
  content: string;
  inProgress?: boolean;
}

interface MessageContentProps {
  text: string;
}

const ThinkingContent: React.FC<ThinkingContentProps> = ({ content, isThinking }) => {
  const [isExpanded, setIsExpanded] = useState<boolean>(false);

  return (
    <Box sx={{ mt: 1 }}>
      <Box 
        sx={{ 
          display: 'flex', 
          alignItems: 'center', 
          gap: 1,
          cursor: 'pointer',
          color: 'text.secondary'
        }}
        onClick={() => setIsExpanded(!isExpanded)}
      >
        <IconButton size="small">
          {isExpanded ? <ChevronUp size={16} /> : <ChevronDown size={16} />}
        </IconButton>
        <Typography variant="caption" sx={{ fontWeight: 500 }}>
          Thinking Process
        </Typography>
        {isThinking && (
          <Box
            component={Loader2}
            size={16}
            className="animate-spin"
            sx={{
              animation: 'spin 1s linear infinite',
              '@keyframes spin': {
                '0%': {
                  transform: 'rotate(0deg)',
                },
                '100%': {
                  transform: 'rotate(360deg)',
                },
              },
            }}
          />
        )}
      </Box>
      <Collapse in={isExpanded}>
        <Box sx={{ 
          pl: 4, 
          pr: 2,
          py: 1,
          mt: 1,
          borderLeft: '2px solid',
          borderColor: 'divider',
          color: 'text.secondary'
        }}>
          <ReactMarkdown
            className="markdown-body"
            rehypePlugins={[rehypeHighlight]}
          >
            {content}
          </ReactMarkdown>
        </Box>
      </Collapse>
    </Box>
  );
};

const MessageContent: React.FC<MessageContentProps> = ({ text }) => {
  const [segments, setSegments] = useState<MessageSegment[]>([]);
  const [isThinking, setIsThinking] = useState<boolean>(false);

  useEffect(() => {
    const parseContent = (): void => {
      const parts: MessageSegment[] = [];
      let remainingText = text;
      
      // Handle <think> tags
      const thinkRegex = /<think>([\s\S]*?)(?:<\/think>|$)/g;
      let lastIndex = 0;
      let match;
      
      while ((match = thinkRegex.exec(text)) !== null) {
        // Add text before the think tag if there is any
        const beforeText = text.slice(lastIndex, match.index).trim();
        if (beforeText) {
          parts.push({
            type: 'text',
            content: beforeText
          });
        }
        
        // Add the thinking content
        parts.push({
          type: 'thinking',
          content: match[1],
          inProgress: !match[0].endsWith('</think>')
        });
        
        lastIndex = match.index + match[0].length;
      }
      
      // Add any remaining text after the last think tag
      const afterText = text.slice(lastIndex).trim();
      if (afterText) {
        parts.push({
          type: 'text',
          content: afterText
        });
      }
      
      setSegments(parts);
      setIsThinking(parts.some(part => part.type === 'thinking' && part.inProgress));
    };

    parseContent();
  }, [text]);

  return (
    <Box>
      {segments.map((segment, index) => {
                  if (segment.type === 'text' && segment.content.trim()) {
          return (
            <Box key={index}>
              <Box sx={{ 
                '& .markdown-body': {
                  whiteSpace: 'pre-wrap',
                  wordBreak: 'break-word',
                  minHeight: '1.5em',
                  lineHeight: 1.6,
                  '& pre': {
                    backgroundColor: (theme) => theme.palette.mode === 'dark' ? '#1e1e1e' : '#f6f8fa',
                    padding: 2,
                    borderRadius: 1,
                    overflow: 'auto'
                  },
                  '& code': {
                    backgroundColor: (theme) => theme.palette.mode === 'dark' ? '#1e1e1e' : '#f6f8fa',
                    padding: '0.2em 0.4em',
                    borderRadius: 1,
                    fontSize: '85%'
                  },
                  '& h1, & h2, & h3, & h4, & h5, & h6': {
                    marginTop: '24px',
                    marginBottom: '16px',
                    fontWeight: 600,
                    lineHeight: 1.25
                  },
                  '& p': {
                    marginTop: '0',
                    marginBottom: '16px'
                  },
                  '& a': {
                    color: (theme) => theme.palette.primary.main,
                    textDecoration: 'none',
                    '&:hover': {
                      textDecoration: 'underline'
                    }
                  },
                  '& img': {
                    maxWidth: '100%',
                    height: 'auto'
                  },
                  '& blockquote': {
                    padding: '0 1em',
                    color: (theme) => theme.palette.text.secondary,
                    borderLeft: (theme) => `0.25em solid ${theme.palette.divider}`,
                    margin: '0 0 16px 0'
                  },
                  '& ul, & ol': {
                    paddingLeft: '2em',
                    marginBottom: '16px'
                  }
                }
              }}>
                <ReactMarkdown
                  className="markdown-body"
                  rehypePlugins={[rehypeHighlight]}
                >
                  {segment.content}
                </ReactMarkdown>
              </Box>
            </Box>
          );
        }
        if (segment.type === 'thinking') {
          return (
            <Box key={index}>
              <ThinkingContent 
                content={segment.content} 
                isThinking={segment.inProgress || false}
              />
            </Box>
          );
        }
        return null;
      })}
    </Box>
  );
};

const MessageList: React.FC<MessageListProps> = ({ messages }) => {
  // Memoize messages to preserve their original timestamps
  const memoizedMessages = React.useMemo(() => messages, [messages]);
  const messageListRef = useRef<HTMLDivElement | null>(null);

  useEffect(() => {
    if (messageListRef.current) {
      messageListRef.current.scrollTop = messageListRef.current.scrollHeight;
    }
  }, [messages]);

  return (
    <Box 
      ref={messageListRef}
      sx={{ 
        flex: 1, 
        overflow: 'auto',
        p: 2,
        display: 'flex',
        flexDirection: 'column',
        gap: 2,
        scrollBehavior: 'smooth',
        '&::-webkit-scrollbar': {
          width: '8px',
          background: 'transparent',
        },
        '&::-webkit-scrollbar-thumb': {
          background: (theme) => theme.palette.divider,
          borderRadius: '4px',
          '&:hover': {
            background: (theme) => theme.palette.action.hover,
          },
        },
      }}
    >
      {memoizedMessages.map((message) => (
        <Box
          key={message.id}
          sx={{
            display: 'flex',
            flexDirection: message.isUser ? 'row-reverse' : 'row',
            alignItems: 'flex-start',
            gap: 2,
            maxWidth: '80%',
            alignSelf: message.isUser ? 'flex-end' : 'flex-start',
          }}
        >
          <Avatar
            src={message.isUser ? '/profiles/user-profile.webp' : '/profiles/ai-profile.webp'}
            alt={message.isUser ? 'User' : 'AI'}
            sx={{ width: 40, height: 40 }}
          />
          <Paper
            elevation={1}
            sx={{
              px: 2,
              py: 1.5,
              flex: 1,
              bgcolor: message.isUser ? 'primary.main' : 'background.paper',
              color: message.isUser ? 'primary.contrastText' : 'text.primary',
              borderRadius: 2,
            }}
          >
            <MessageContent text={message.text} />
            {message.sources && message.sources.length > 0 && (
              <Box sx={{ mt: 2, pt: 1, borderTop: '1px solid', borderColor: 'divider' }}>
                <Typography variant="caption" color="text.secondary">
                  Sources:
                </Typography>
                {message.sources.map((source, index) => (
                  <Typography
                    key={index}
                    variant="caption"
                    component="div"
                    color="text.secondary"
                  >
                    {source.path}
                  </Typography>
                ))}
              </Box>
            )}
          </Paper>
        </Box>
      ))}
    </Box>
  );
};

export default MessageList;