import React, { useState } from 'react';
import { 
  Button, 
  Dialog, 
  DialogTitle, 
  DialogContent, 
  DialogActions, 
  List, 
  ListItem, 
  ListItemText, 
  ListItemIcon,
  CircularProgress,
  Alert,
  Box,
  Typography,
  TextField,
  InputAdornment,
} from '@mui/material';
import { 
  Folder as FolderIcon,
  AddCircleOutline as AddIcon,
  Google as GoogleIcon,
  Cloud as DropboxIcon,
  Chat as DiscordIcon,
  Computer as LocalIcon,
  Search,
  Padding
} from '@mui/icons-material';

import { useElectron } from '../../hooks/useElectron';

interface Directory {
  name: string;
  path: string;
}

interface DirectoryPickerProps {
  onSelect: (dirPath: string) => void;
}

export default function DirectoryPicker({ onSelect }: DirectoryPickerProps) {
  const electron = useElectron();
  const [open, setOpen] = useState(false);
  const [directories, setDirectories] = useState<Directory[]>([]);
  const [currentPath, setCurrentPath] = useState<string>('');
  const [isLoading, setIsLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);

  const handleOpen = async () => {
    try {
      setOpen(true);
      setIsLoading(true);
      setError(null);
      const homeResponse = await electron.getUserHome();
      if (!homeResponse.success || !homeResponse.data) {
        throw new Error(homeResponse.error || 'Failed to get user home directory');
      }
      const homePath = homeResponse.data;
      setCurrentPath(homePath);
      
      const dirResponse = await electron.listDirectories(homePath);
      if (!dirResponse.success || !dirResponse.data) {
        throw new Error(dirResponse.error || 'Failed to load directories');
      }
      setDirectories(dirResponse.data);
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Failed to load directories');
      console.error('Error loading directories:', err);
    } finally {
      setIsLoading(false);
    }
  };

  const handleClose = () => {
    setOpen(false);
    setError(null);
  };

  const handleDirectoryClick = async (dirPath: string) => {
    try {
      setIsLoading(true);
      setError(null);
      setCurrentPath(dirPath);
      const dirResponse = await electron.listDirectories(dirPath);
      if (!dirResponse.success || !dirResponse.data) {
        throw new Error(dirResponse.error || 'Failed to load directories');
      }
      setDirectories(dirResponse.data);
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Failed to load directories');
      console.error('Error loading directories:', err);
    } finally {
      setIsLoading(false);
    }
  };

  const handleSelect = () => {
    onSelect(currentPath);
    handleClose();
  };

  return (
    <>
      <Box>
        <Box sx={{ 
          position: 'relative',
          display: 'flex',
          justifyContent: 'flex-end',
          mt: 1.5,
          mr: 2,
          mb: 2,
        }}>

    <TextField
      fullWidth
      variant="outlined"
      size="small"
      placeholder="Search..."
      InputProps={{
        startAdornment: (
          <InputAdornment position="start">
            <Search className="text-gray-400"/>

          </InputAdornment>
        ),
        sx: {ml:2,
          '&::-webkit-scrollbar': {
            width: '8px',
            background: 'transparent',
          },
          '&::-webkit-scrollbar-thumb': {
            background: '#2196f3',
            borderRadius: '2px',
            '&:hover': {
              background: '#64b5f6',
            },
          },
          '&::-webkit-scrollbar-track': {
            background: 'transparent',
          }
        }
      }}
    />          <Button 
            variant="contained" 
            onClick={handleOpen}
            size="small"
            sx={{ 
              ml:3,
              minWidth: 0,
              width: '64px',
              height: '38px',
              borderRadius: '5%',
              padding: 0,
              overflow: 'hidden',
              backgroundColor: 'primary.main',
              transition: 'all 0.3s ease',
              '& .MuiButton-startIcon': {
                margin: 0,
              },
              '& .buttonText': {
                width: 0,
                overflow: 'hidden',
                transition: 'width 0.3s ease',
                whiteSpace: 'nowrap',
              },
              '&:hover': {
                backgroundColor: 'primary.dark',
                width: '400%',
                borderRadius: '5%',
                '& .buttonText': {
                  width: '150px',
                  marginLeft: '8px',
                },
              },
            }}
          >
            <AddIcon sx={{ fontSize: 20 }} />
            <span className="buttonText">Add New Folder</span>
          </Button>
        </Box>
      </Box>

      <Dialog 
        open={open} 
        onClose={handleClose}
        maxWidth="md"
        fullWidth
      >
        <DialogTitle>Select Directory</DialogTitle>
        <DialogContent>
          {error && (
            <Alert severity="error" sx={{ mb: 2 }}>
              {error}
            </Alert>
          )}
          {isLoading ? (
            <div style={{ display: 'flex', justifyContent: 'center', padding: 20 }}>
              <CircularProgress />
            </div>
          ) : (
            <List>
              {currentPath && (
                <ListItem 
                  button 
                  onClick={() => {
                    const parentPath = currentPath.split(/[/\\]/).slice(0, -1).join('/');
                    if (parentPath) {
                      handleDirectoryClick(parentPath);
                    }
                  }}
                >
                  <ListItemIcon>
                    <FolderIcon />
                  </ListItemIcon>
                  <ListItemText primary=".." secondary="Go up" />
                </ListItem>
              )}
              {directories.map((dir) => (
                <ListItem 
                  key={dir.path} 
                  button 
                  onClick={() => handleDirectoryClick(dir.path)}
                >
                  <ListItemIcon>
                    <FolderIcon />
                  </ListItemIcon>
                  <ListItemText 
                    primary={dir.name} 
                    secondary={dir.path}
                  />
                </ListItem>
              ))}
            </List>
          )}
        </DialogContent>
        <DialogActions>
          <Button onClick={handleClose}>Cancel</Button>
          <Button 
            onClick={handleSelect} 
            variant="contained"
            disabled={!currentPath}
          >
            Select Directory
          </Button>
        </DialogActions>
      </Dialog>
    </>
  );
}
