import React from 'react';
import { Box, Typography, List, ListItem, ListItemText, ListItemIcon, IconButton, Tooltip } from '@mui/material';
import { Folder as FolderIcon, Psychology as LLMIcon, ImageSearch as OCRIcon } from '@mui/icons-material';
import { DocumentMetadata } from '../../../electron/types';
import DirectoryPicker from './DirectoryPicker';
import { useFileSystem } from '../../hooks/useFileSystem';

interface FileMetadata {
  [path: string]: DocumentMetadata;
}

export default function FileExplorer() {
  const { watchedDirectories, watchDirectory, unwatchDirectory } = useFileSystem();
  const [fileMetadata, setFileMetadata] = React.useState<FileMetadata>({});

  React.useEffect(() => {
    // Get document metadata from electron store
    window.electron.getDocuments().then((response) => {
      if (response.success && response.data) {
        const metadata: FileMetadata = {};
        response.data.forEach(doc => {
          metadata[doc.path] = doc;
        });
        setFileMetadata(metadata);
      }
    });
  }, [watchedDirectories]);

  const handleDirectorySelect = async (dirPath: string) => {
    if (!watchedDirectories.includes(dirPath)) {
      await watchDirectory(dirPath);
    }
  };

  const handleDirectoryRemove = async (dirPath: string) => {
    await unwatchDirectory(dirPath);
  };

  return (
    <Box sx={{ 
      display: 'flex', 
      flexDirection: 'column',
      height: '100%',
      overflow: 'hidden'
    }}>

      <DirectoryPicker onSelect={handleDirectorySelect} />

      <Box sx={{ flex: 1, overflow: 'auto' }}>
        <List>
          {watchedDirectories.map((dir) => (
            <ListItem
              key={dir}
              secondaryAction={
                <IconButton 
                  edge="end" 
                  size="small"
                  onClick={() => handleDirectoryRemove(dir)}
                >
                  ×
                </IconButton>
              }
            >
              <ListItemIcon sx={{ display: 'flex', gap: 0.5 }}>
                <FolderIcon />
                {fileMetadata[dir]?.hasEmbeddings && (
                  <Tooltip title="LLM Embeddings Enabled">
                    <LLMIcon color="primary" fontSize="small" />
                  </Tooltip>
                )}
                {fileMetadata[dir]?.hasOcr && (
                  <Tooltip title="OCR Enabled">
                    <OCRIcon color="secondary" fontSize="small" />
                  </Tooltip>
                )}
              </ListItemIcon>
              <ListItemText
                primary={dir.split(/[/\\]/).pop()} 
                secondary={dir}
                secondaryTypographyProps={{
                  sx: { 
                    overflow: 'hidden',
                    textOverflow: 'ellipsis',
                    whiteSpace: 'nowrap'
                  }
                }}
              />
            </ListItem>
          ))}
        </List>
      </Box>
    </Box>
  );
}
