import React, { useState, useEffect } from 'react';
import {
  Button,
  TextField,
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  CircularProgress,
  Alert,
  Box,
  Typography,
  Paper,
  Divider,
  Switch,
  FormControlLabel,
} from '@mui/material';
import { useLLMConfig } from '../../hooks/useLLMConfig';
import { useOllamaModels } from '../../hooks/useOllamaModels';
import type { LLMConfig } from '../../../electron/types';

const defaultConfig: LLMConfig = {
  provider: 'ollama',
  model: 'jimscard/blackhat-hacker:v2',
  baseUrl: 'http://localhost:11434',
  temperature: 0.7,
  apiKey: undefined
};

export default function SettingsPanel() {
  const { config, isLoading, error, updateConfig, reloadConfig } = useLLMConfig();
  const ollamaModels = useOllamaModels();
  const [formData, setFormData] = useState<LLMConfig>(defaultConfig);
  const [isSaving, setIsSaving] = useState(false);

  // Reload config when component becomes visible
  useEffect(() => {
    const handleVisibilityChange = () => {
      if (document.visibilityState === 'visible') {
        reloadConfig();
      }
    };

    document.addEventListener('visibilitychange', handleVisibilityChange);
    return () => {
      document.removeEventListener('visibilitychange', handleVisibilityChange);
    };
  }, [reloadConfig]);

  useEffect(() => {
    if (!config) {
      setFormData(defaultConfig);
      return;
    }

    // Initialize form data with loaded config, only using defaults for missing values
    setFormData({
      provider: config.provider,
      model: config.model || defaultConfig.model,
      baseUrl: config.baseUrl || defaultConfig.baseUrl,
      temperature: config.temperature ?? defaultConfig.temperature,
      apiKey: config.apiKey
    });
  }, [config]);

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    try {
      setIsSaving(true);
      await updateConfig(formData);
    } catch (error) {
      console.error('Failed to save settings:', error);
    } finally {
      setIsSaving(false);
    }
  };

  if (isLoading) {
    return (
      <Box sx={{ display: 'flex', justifyContent: 'center', p: 3 }}>
        <CircularProgress />
      </Box>
    );
  }

  return (
    <Box sx={{ 
      p: 3, 
      height: '100%', 
      overflow: 'auto',
      display: 'flex',
      flexDirection: 'column',
      flex: 1,
      minHeight: 0
    }}>
      {/* Search Settings */}
      <Paper  sx={{ p: 3, mb: 3 }} >
        <Typography variant="h6" gutterBottom>Scan Settings</Typography>
        <FormControlLabel
          control={<Switch defaultChecked />}
          label="Include file contents in search"
          sx={{ mb: 2, display: 'block' }}
        />
        <FormControlLabel
          control={<Switch defaultChecked />}
          label="Include file names in search"
          sx={{ mb: 2, display: 'block' }}
        />
        <FormControlLabel
          control={<Switch defaultChecked />}
          label="Use OCR to extract text from images/pdfs?"
          sx={{ mb: 2, display: 'block' }}
        />
        <FormControlLabel
          control={<Switch defaultChecked />}
          label="Unzip files to check contents?"
          sx={{ mb: 2, display: 'block' }}
        />
        <TextField
          fullWidth
          label="File extensions to ignore"
          placeholder="e.g., .git, .env, node_modules"
          margin="normal"
        />
      </Paper>
      
      {/* Search Settings */}
      <Paper  sx={{ p: 3, mb: 3 }} >
        <Typography variant="h6" gutterBottom>Connections</Typography>
        <FormControlLabel
          control={<Switch disabled />}
          label="Google Drive"
          sx={{ mb: 2, display: 'block' }}
        />
        <FormControlLabel
          control={<Switch disabled />}
          label="Dropbox"
          sx={{ mb: 2, display: 'block' }}
        />
        <FormControlLabel
          control={<Switch disabled />}
          label="Discord"
          sx={{ mb: 2, display: 'block' }}
        />
      </Paper>
      
      {/* LLM Settings */}
      <Paper sx={{ p: 3, mb: 3 }}>
        <Typography variant="h6" gutterBottom>LLM Settings</Typography>
        <form onSubmit={handleSubmit}>
          {error && (
            <Alert severity="error" sx={{ mb: 2 }}>
              {error}
            </Alert>
          )}
          <FormControl fullWidth margin="normal">
            <InputLabel>Provider</InputLabel>
            <Select
              value={formData.provider || defaultConfig.provider}
              label="Provider"
              onChange={(e) => setFormData(prev => ({
                ...prev,
                provider: e.target.value as LLMConfig['provider']
              }))}
            >
              <MenuItem value="openai">OpenAI</MenuItem>
              <MenuItem value="openrouter">OpenRouter</MenuItem>
              <MenuItem value="ollama">Ollama</MenuItem>
            </Select>
          </FormControl>

          {formData.provider !== 'ollama' && (
            <TextField
              fullWidth
              margin="normal"
              label="API Key"
              type="password"
              value={formData.apiKey ?? ''}
              onChange={(e) => setFormData(prev => ({
                ...prev,
                apiKey: e.target.value
              }))}
            />
          )}

          {formData.provider === 'ollama' ? (
            <FormControl fullWidth margin="normal">
              <InputLabel>Model</InputLabel>
              <Select
                value={formData.model || defaultConfig.model}
                label="Model"
                onChange={(e) => setFormData(prev => ({
                  ...prev,
                  model: e.target.value
                }))}
                displayEmpty
              >
                <MenuItem value="jimscard/blackhat-hacker:v2">jimscard/blackhat-hacker:v2</MenuItem>
                {ollamaModels.models.map((model) => (
                  model !== 'jimscard/blackhat-hacker:v2' && <MenuItem key={model} value={model}>{model}</MenuItem>
                ))}
              </Select>
              {ollamaModels.error && (
                <Alert severity="error" sx={{ mt: 1 }}>
                  {ollamaModels.error}
                </Alert>
              )}
            </FormControl>
          ) : (
            <TextField
              fullWidth
              margin="normal"
              label="Model"
              value={formData.model ?? defaultConfig.model}
              onChange={(e) => setFormData(prev => ({
                ...prev,
                model: e.target.value
              }))}
            />
          )}

          {formData.provider === 'ollama' && (
            <TextField
              fullWidth
              margin="normal"
              label="Base URL"
              value={formData.baseUrl ?? defaultConfig.baseUrl}
              onChange={(e) => setFormData(prev => ({
                ...prev,
                baseUrl: e.target.value
              }))}
            />
          )}

          <TextField
            fullWidth
            margin="normal"
            label="Temperature"
            type="number"
            inputProps={{ min: 0, max: 1, step: 0.1 }}
            value={formData.temperature ?? defaultConfig.temperature}
            onChange={(e) => setFormData(prev => ({
              ...prev,
              temperature: parseFloat(e.target.value)
            }))}
          />
          <Box sx={{ mt: 3 }}>
            <Button 
              type="submit" 
              variant="contained" 
              disabled={isSaving}
              fullWidth
            >
              {isSaving ? <CircularProgress size={24} /> : 'Save LLM Settings'}
            </Button>
          </Box>
        </form>
      </Paper>
    </Box>
  );
}
