import { useState, useEffect, useCallback } from 'react';
import { useElectron } from './useElectron';
import type { LLMConfig } from '../../electron/types';

export function useLLMConfig() {
  const electron = useElectron();
  const [config, setConfig] = useState<LLMConfig | null>(null);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    loadConfig();
  }, []);

  const loadConfig = async () => {
    try {
      setIsLoading(true);
      setError(null);
      const config = await electron.getLLMConfig().catch(err => {
        console.error('Error loading LLM config:', err);
        return null;
      });
      setConfig(config || {
        provider: 'ollama',
        model: 'llama2:7b',
        baseUrl: 'http://localhost:11434',
        temperature: 0.7,
        apiKey: undefined
      });
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Failed to load LLM config');
      console.error('Error loading LLM config:', err);
    } finally {
      setIsLoading(false);
    }
  };

  const updateConfig = useCallback(async (newConfig: LLMConfig) => {
    try {
      setIsLoading(true);
      setError(null);
      await electron.updateLLMConfig(newConfig);
      setConfig(newConfig);
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Failed to update LLM config');
      console.error('Error updating LLM config:', err);
      throw err;
    } finally {
      setIsLoading(false);
    }
  }, [electron]);

  return {
    config,
    isLoading,
    error,
    updateConfig,
    reloadConfig: loadConfig,
  };
}
